package asw.jms.simpleasynchconsumer;

import javax.jms.*;

import asw.util.cancellable.Cancellable;
import asw.util.logging.AswLogger;
import asw.util.sleep.Sleeper;

import java.util.logging.Logger;

/*
 * SimpleAsynchConsumer.java
 *
 * Un SimpleAsynchConsumer  un endpoint per ricevere messaggi
 * da una generica destinazione JMS (coda o argomento).
 * 
 * La ricezione dei messaggi avviene secondo la modalit asincrona.
 *
 * Delega la ricezione dei messaggi ricevuti ad un MessageListener
 * associato a questo consumer.
 *
 * I messaggi potranno essere inviati da SimpleProducer.
 *
 * Variante di un esempio nel tutorial per Java EE.
 *
 * @author Luca Cabibbo
 */
public class SimpleAsynchConsumer implements Cancellable {

	/* logger */
	private static Logger logger = AswLogger.getInstance().getLogger("asw.jms.simpleasynchconsumer");

	/* nome di questo consumer */
    private String name;
    /* destinazione di questo consumer */
    private Destination destination;
    /* connection factory di questo consumer */
    private ConnectionFactory connectionFactory;

    /* per il callback dei messaggi ricevuti */
    private MessageListener listener;

    /* contesto jms */
    private JMSContext context = null;
    /* per la ricezione di messaggi da destination */
    JMSConsumer messageConsumer = null;

    /*  stato cancellato? */
    private boolean cancelled;

    /**
     * Crea un nuovo SimpleAsynchConsumer, di nome n, per una destinazione di nome dn.
     */
    public SimpleAsynchConsumer(String n, Destination d, ConnectionFactory cf, MessageListener listener) {

        this.name = n;
        this.destination = d;
        this.connectionFactory = cf;
        this.listener = listener;
        this.cancelled = false;
    }

    /**
     * Apre la connessione alla destinazione JMS di questo consumer.
     */
    public void connect() {
    	logger.fine(this.toString() + ": Connecting...");
        context = connectionFactory.createContext();
    	messageConsumer = context.createConsumer(destination);
    	logger.info(this.toString() + ": Connected");
    }

    /**
     * Si disconnette dalla destinazione JMS di questo consumer.
     */
    public void disconnect() {
        if (context != null) {
        	logger.fine(this.toString() + ": Disconnecting...");
            context.close();
            context = null;
        	logger.info(this.toString() + ": " + this.listener.toString());
        	logger.info(this.toString() + ": Disconnected");
        }
    }

    /**
     * Riceve infiniti messaggi dalla destinazione (sessione di ricezione)
     * di questo consumer.
     */
    public void receiveMessages() {
		messageConsumer.setMessageListener(this.listener);
        /* avvia la consegna di messaggi per la connessione */
		context.start();
		/* va avanti, fino a quando non viene fermato */
		while (!isCancelled()) {
			Sleeper.sleep(10);
		}
        /*  stato fermato, arresta la consegna di messaggi per la connessione */
        context.stop();
    }

    /**
     * Cancella questo consumer.
     */
    public void cancel() {
    	this.cancelled = true;
    }

    /**
     * Questo consumer  stato cancellato?
     */
    public boolean isCancelled() {
    	return this.cancelled;
    }

    /**
     * Retituisce il nome di questo consumer. 
     */
    public String getName() {
    	return name; 
    }
    
    /**
     * Restituisce una descrizione di questo consumer.
     */
    public String toString() {
        return "SimpleAsynchConsumer[" +
                "name=" + name +
//                ", destination=" + destination.toString() +
                "]";
    }

}
